#ifndef LAYER_H_INCLUDED
#define LAYER_H_INCLUDED

#include "Pokitto.h"

class Layer
{



private:
    int16_t xcell;
    int16_t ycell;
    int8_t xcellRem;
    int8_t ycellRem;
    int8_t frame=0;
    int8_t frameSub=0;
    int8_t frameMax;
    int8_t idAnim;


public:
    Layer()
    {

    }

    void update()
    {
        frameSub++;
        if(frameSub>10)
        {
            frame++;
            frameSub=0;
        }

    }

    //This kind of magic thanks to @Pharap the C++ Guru
    template<std::size_t col, std::size_t row, std::size_t lenA,std::size_t lenD>
    void drawMap(Camera *cam, const uint8_t (&mapData)[col][row],
                 const std::bitset<4> (&mapRenders)[col][row],
                 const std::bitset<8> (&mapProp)[col][row],
                 const uint8_t (&mapAnim)[lenA],
                 uint8_t (&mapDestr)[lenD])
    {
        uint16_t id;
        std::bitset<4> tr;
        std::bitset<8> tp;

        //Calculate
        xcell = (cam->getX()/cam->getZ())/CELL_WIDTH;
        ycell = (cam->getY()/cam->getZ())/CELL_HEIGHT;

        xcellRem = (cam->getX()/cam->getZ())%CELL_WIDTH;
        ycellRem = (cam->getY()/cam->getZ())%CELL_HEIGHT;

        short cc=0;
        short rr=ycell;
        bool alive=true;
        for(short r=0; r<12; r++,rr++)
        {
            cc=xcell;
            for(short c=0; c<16; c++,cc++)
            {
                //
                alive=true;

                //Sanity check
                if(cc>=0 && rr >=0)
                {
                    id=mapData[rr][cc];
                    tr=mapRenders[rr][cc];
                    tp=mapProp[rr][cc];

                    //Distructible
                    if(tp[3])
                    {
                        //Find destructible tile life
                        for(uint16_t i=0; i<lenD; i+=3)
                        {
                            if(mapDestr[i]==cc && mapDestr[i+1]==rr)//x && y
                            {
                                if(mapDestr[i+2]<1)
                                    alive=false;
                            }
                        }
                    }

                    if(alive)
                    {
                        //Flipped??
                        if(tr[0])
                            d.drawBitmap((c*8)-xcellRem,(r*8)-ycellRem,tiles[id],0,1);
                        else
                            d.drawBitmap((c*8)-xcellRem,(r*8)-ycellRem,tiles[id]);

                        //Animated??
                        if(tp[4])
                        {
                            //Find animation
                            for(uint16_t i=0; i<lenA; i+=5)
                            {
                                if(mapAnim[i]==cc && mapAnim[i+1]==rr)//x && y
                                {
                                    frameMax=mapAnim[i+3];
                                    idAnim = frame % frameMax;
                                    if(idAnim==mapAnim[i+2])
                                    {
                                        d.drawBitmap((c*8)-xcellRem,(r*8)-ycellRem,tiles[mapAnim[i+4]]);
                                    }
                                }
                            }
                        }
                    }
                }

            }
        }
    }

};


#endif // LAYER_H_INCLUDED
