#ifndef PLAYER_H_INCLUDED
#define PLAYER_H_INCLUDED

#include "Pokitto.h"
#include "Particle.h"
#include "Engine.h"

#define SPEED_RUN 2.5
#define FRICTION_RUN 0.55

#define SPEED_JUMP 4.5
#define ACC_FALL 0.4
#define SPEED_MAX_FALL 4.5
#define SPEED_SHOOT 4.0


class Player
{

    typedef enum
    {
        Idle,
        Run,
        Jump,
        Fall,
        Fire,
        Crunch
    } State;

private:
    const Fix16 speedRun = Fix16(SPEED_RUN);
    const Fix16 frictionRun = Fix16(FRICTION_RUN);
    const Fix16 speedJUMP = Fix16(SPEED_JUMP);
    const Fix16 accFALL = Fix16(ACC_FALL);
    const Fix16 speedMaxFALL = Fix16(SPEED_MAX_FALL);
    const Fix16 speedShoot = Fix16(SPEED_SHOOT);
    const Fix16 angleShoot = 2*PI_DIV_4;

    Fix16 x;
    Fix16 y;
    int16_t xcell;
    int16_t ycell;
    int16_t xscreen;
    int16_t yscreen;

    Fix16 speedX;
    Fix16 speedY;

    State state=State::Idle;
    bool direction=false;


    bool onSolid=false;
    bool uwSolid=false;
    bool dwSolid=false;
    bool sxSolid=false;
    bool dxSolid=false;

    bool dwAlive=false;



public:

    Player(int16_t x,int16_t y)
    {
        this->x=fix16_from_int(x);
        this->y=fix16_from_int(y);
        speedX=0.0;
        speedY=0.0;
    }

    int getX()
    {
        return fix16_to_int(x);
    }

    int getY()
    {
        return fix16_to_int(y);
    }

    template<std::size_t col, std::size_t row,std::size_t lenD>
    void update(const std::bitset<8>(&mapProp)[col][row],uint8_t (&mapDestr)[lenD],Engine* engine)
    {
        if(g.leftBtn())
            speedX=speedX-speedRun;
        if(g.rightBtn())
            speedX=speedX+speedRun;

        speedX=speedX*frictionRun;

        if(g.aBtn() && speedY==0.0)
        {
            speedY-=speedJUMP;
        }

        if(g.bBtn())
        {
            Particle p=Particle(x+fix16_from_int(4),
                                y+fix16_from_int(4),
                                speedX+speedShoot,
                                angleShoot);
            engine->AddParticle(p);
        }

        //Calculate tile we are
        xcell = (getX()+CELL_HALF_WIDTH)/CELL_WIDTH;
        ycell = getY()/CELL_HEIGHT;

        //if destructible
        dwAlive=true;
        if(mapProp[ycell+1][xcell][3])
        {
            //Find destructible tile life
            for(uint16_t i=0; i<lenD; i+=3)
            {
                if(mapDestr[i]==xcell && mapDestr[i+1]==(ycell+1))//x && y
                {
                    if(mapDestr[i+2]<1)
                        dwAlive=false;
                }
            }
        }

        //Check solids
        onSolid=mapProp[ycell][xcell][0];
        uwSolid=mapProp[ycell-1][xcell][0];
        dwSolid=dwAlive && mapProp[ycell+1][xcell][0];
        sxSolid=mapProp[ycell][xcell-1][0];
        dxSolid=mapProp[ycell][xcell+1][0];

        //Apply force
        x+=speedX;
        y+=speedY;


        //Collisions
        //Verify we are not inside a solid
//        if(onSolid)
//        {
//            x=fix16_from_int((xcell)*CELL_WIDTH);
//            y=fix16_from_int(ycell*CELL_HEIGHT);
//        }

        if(dwSolid && speedY>0.0)
        {
            speedY=0.0;
            y=fix16_from_int(ycell*CELL_HEIGHT);
        }
        else
        {
            if(speedY<speedMaxFALL)
                speedY+=accFALL;
        }
        //
        if(sxSolid && speedX<0.0)
        {
            speedX=0.0;
            x=fix16_from_int((xcell)*CELL_WIDTH);
        }

        if(dxSolid && speedX>0.0)
        {
            speedX=0.0;
            x=fix16_from_int((xcell)*CELL_WIDTH);
        }


    }

    void draw(Camera *cam)
    {
        xscreen=(getX()-cam->getX());
        yscreen=(getY()-cam->getY());

        if(dwSolid)
            d.color=9;
        d.fillCircle(xscreen+4,yscreen+4,3);


//       d.color=4;
//       d.drawRect(xscreen,yscreen,8,8);

    }

};


#endif // PLAYER_H_INCLUDED
